/*
   Move the definitions of the Comparator
   classes into this file as private classes.
*/
import java.util.*;

public class ComparatorTester_1_fn
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));


      // Call the sort() method and pass it an anonymous comparator object.

      Collections.sort(countries, new CountryComparatorByName(new FnReverse(false)));
      // Now the List is sorted by country name, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByName(new FnReverse(true)));
      // Now the List is sorted by country name, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByArea(new FnLT()));
      // Now the List is sorted by area, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByArea(new FnGT()));
      // Now the List is sorted by area, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByPopulation(new FnLT()));
      // Now the List is sorted by population, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByPopulation(new FnGT()));
      // Now the List is sorted by population, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}


/**
   Create "functors" that hold the comparison operations, < or >.
*/
abstract class Fn  // create a "functor" type
{
   public abstract boolean fn(double x, double y);
}

class FnLT extends Fn  // create a sub class of the abstract functor
{
   public boolean fn(double x, double y)
   {
      return x < y;
   }
}

class FnGT extends Fn  // create a sub class of the abstract functor
{
   public boolean fn(double x, double y)
   {
      return x > y;
   }
}

/**
   Create a functor that can reverse a comparator.
*/
class FnReverse
{
   private int multiplier;

   public FnReverse(boolean doReverse)
   {
      this.multiplier = (doReverse) ? -1 : +1;
   }

   public int fn(int n)
   {
      return multiplier * n;
   }
}


/**
  These comparators are now private classes (they are not inner classes).
*/
class CountryComparatorByName implements Comparator<Country>
{
   FnReverse fn;

   CountryComparatorByName(FnReverse fn)
   {
      this.fn = fn;
   }

   public int compare(Country country1, Country country2)
   {
      return fn.fn( country1.getName().compareTo(country2.getName() ));
   }
}

class CountryComparatorByArea implements Comparator<Country>
{
   Fn fn; // functor that holds <, >, <=, or >=

   CountryComparatorByArea(Fn fn)
   {
      this.fn = fn;
   }

   public int compare(Country country1, Country country2)
   {
      if ( fn.fn(country1.getArea(), country2.getArea()) ) return -1;
      if (!fn.fn(country1.getArea(), country2.getArea()) ) return  1;
      return 0;
   }
}

class CountryComparatorByPopulation implements Comparator<Country>
{
   Fn fn; // functor that holds <, >, <=, or >=

   CountryComparatorByPopulation(Fn fn)
   {
      this.fn = fn;
   }

   public int compare(Country country1, Country country2)
   {
      if ( fn.fn(country1.getPopulation(), country2.getPopulation()) ) return -1;
      if (!fn.fn(country1.getPopulation(), country2.getPopulation()) ) return  1;
      return 0;
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   public String getName()
   {
      return this.name;
   }

   public double getArea()
   {
      return this.area;
   }

   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}
