/*
   Move the definitions of the Comparator classes into
   the body of the main() method as nested, inner classes.
*/
import java.util.*;

public class ComparatorTester_2
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));


      // These comparators are now (named) inner classes.
      class CountryComparatorByName_a implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            return country1.getName().compareTo(country2.getName());
         }
      }

      class CountryComparatorByName_d implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            return country2.getName().compareTo(country1.getName());
         }
      }

      class CountryComparatorByArea_a implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            if (country1.getArea() < country2.getArea()) return -1;
            if (country1.getArea() > country2.getArea()) return  1;
            return 0;
         }
      }

      class CountryComparatorByArea_d implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            if (country1.getArea() > country2.getArea()) return -1;
            if (country1.getArea() < country2.getArea()) return  1;
            return 0;
         }
      }

      class CountryComparatorByPopulation_a implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            if (country1.getPopulation() < country2.getPopulation()) return -1;
            if (country1.getPopulation() > country2.getPopulation()) return  1;
            return 0;
         }
      }

      class CountryComparatorByPopulation_d implements Comparator<Country>
      {
         public int compare(Country country1, Country country2)
         {
            if (country1.getPopulation() > country2.getPopulation()) return -1;
            if (country1.getPopulation() < country2.getPopulation()) return  1;
            return 0;
         }
      }

      // Call the sort() method and pass it an anonymous comparator object.

      Collections.sort(countries, new CountryComparatorByName_a());
      // Now the List is sorted by country name, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByName_d());
      // Now the List is sorted by country name, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByArea_a());
      // Now the List is sorted by area, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByArea_d());
      // Now the List is sorted by area, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByPopulation_a());
      // Now the List is sorted by population, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, new CountryComparatorByPopulation_d());
      // Now the List is sorted by population, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   public String getName()
   {
      return this.name;
   }

   public double getArea()
   {
      return this.area;
   }

   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}
