/*
   Create a parameterized comparator that
   is as close to Scheme's as possible.
*/
import java.util.*;
import java.util.function.*;

public class ComparatorTester_Java8_v1
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));

      // Give nice names to all the lambdas that we need.
      // The types all come from java.util.function
      BiPredicate<Double, Double> lt = (x,y) -> x < y;
      BiPredicate<Double, Double> gt = (x,y) -> x > y;
      ToDoubleFunction<Country> selectA = Country::getArea;
      ToDoubleFunction<Country> selectP = Country::getPopulation;

      // This class does what the makeCmp function does in Scheme
      // (but makeCmp uses a closure to remember all its parameters).
      class CmpCountries implements Comparator<Country>
      {  // these parameters are all "functors"
         private BiPredicate<Double, Double> cmp;
         private ToDoubleFunction<Country> s;

         CmpCountries(BiPredicate<Double, Double> cmp, ToDoubleFunction<Country> s)
         {  this.cmp = cmp;
            this.s = s;
         }

         public int compare(Country c1, Country c2)
         {  if (cmp.test(s.applyAsDouble(c1), s.applyAsDouble(c2))) return -1;
            if (cmp.test(s.applyAsDouble(c2), s.applyAsDouble(c1))) return  1;
            return 0;
         }
      }//CmpCountries


      // Call the sort() method and pass it a (parameterized) comparator object.

      countries.sort(new CmpCountries(lt, selectA));
      countries.forEach(System.out::println);System.out.println();

      countries.sort(new CmpCountries(gt, selectA));
      countries.forEach(System.out::println);System.out.println();

      countries.sort(new CmpCountries(lt, selectP));
      countries.forEach(System.out::println);System.out.println();

      countries.sort(new CmpCountries(gt, selectP));
      countries.forEach(System.out::println);System.out.println();
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   public String getName()
   {
      return this.name;
   }

   public double getArea()
   {
      return this.area;
   }

   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}
