/*
   Create a parameterized comparator that
   is as close to Scheme's as possible.
*/
import java.util.*;

public class ComparatorTester_Java8_v2
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));


      // Use a "lambda expression" in place of a "comparator object".
      // Call the sort() method and pass it an anonymous lambda expression
      // that implements the one method from the Comparator interface.
      Collections.sort(countries, (c1, c2) -> c1.getName().compareTo(c2.getName()));
      // Now the List is sorted by country name, ascending.
      // Print the results using a lambda expression (that uses "type inference").
      countries.forEach(c -> System.out.println(c));
      System.out.println();

      // sort() has been added to the List class, so we can call it this way.
      countries.sort((c1, c2) -> c2.getName().compareTo(c1.getName()));
      // Now the List is sorted by country name, descending.
      // Print the results in a very concise way by using a "method reference".
      countries.forEach(System.out::println);
      System.out.println();


      countries.sort((c1,c2) -> c1.getArea().compareTo(c2.getArea()));
      // Now the List is sorted by area, ascending.
      countries.forEach(System.out::println);
      System.out.println();

      countries.sort((c1,c2) -> c2.getArea().compareTo(c1.getArea()));
      // Now the List is sorted by area, descending.
      countries.forEach(System.out::println);
      System.out.println();

      countries.sort((c1,c2) -> c1.getPopulation().compareTo(c2.getPopulation()));
      // Now the List is sorted by population, ascending.
      countries.forEach(System.out::println);
      System.out.println();

      countries.sort((c1,c2) -> c2.getPopulation().compareTo(c1.getPopulation()));
      // Now the List is sorted by population, descending.
      countries.forEach(System.out::println);
      System.out.println();
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private Double area;  // changed to Double
   private Double population; // changed to Double

   public Country(String name, double area, double population)
   {
      this.name = name; // uses autoboxing
      this.area = area; // uses autoboxing
      this.population = population;
   }

   public String getName()
   {
      return this.name;
   }

   public Double getArea()
   {
      return this.area;
   }

   public Double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}
